package framework.config;

import java.io.*;
import java.util.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Config class is a singleton properties collection
 * that simplifies access to property file based configuration.
 * Applications must load config before trying to access init parameters.
 */
public class Config {

    //
    // Singleton
    //

    /**
     * Single instance created upon class loading.
     */
    private static final Config fINSTANCE = new Config();

    /**
     * Return singleton instance of config.
     */
    public static synchronized Config getInstance() {
        return fINSTANCE;
    }

    /**
     * Private constructor prevents construction outside this class.
     */
    private Config() {
        log.debug("Config constructor invoked");
        _props = null;
    }

    //
    // Members
    //
    /** Default property file resource path */
    public static final String DEFAULT_PROPERTY_FILE_RESOURCE_PATH = "/config.properties";

    /** Global properties object */
    private Properties _props;

    /** Logging */
    private Log log = LogFactory.getLog(Config.class);


    //
    // Methods
    //

    /**
     * Reads a configuration parameter.
     * @param key string that identifies the parameter
     * @return parameter value or null if parameter not found
     */
    public synchronized String getInitParameter(String key) {
        if(_props == null)
            throw new IllegalStateException("Configuration must be loaded before initialization parameters can be accessed!");
        return _props.getProperty(key);
    }

    /**
     * Returns a shallow clone of the configuration parameters
     * @return properties collection
     */
    public synchronized Properties getInitProperties() {
        if(_props == null)
            return null;
        else
            return (Properties) _props.clone();
    }

    /**
     * Clears the configuration
     */
    public synchronized void reset() {
        _props = null;
    }

    /**
     * Create new properties object from resource file.
     * @throws ConfigException when resource is not found
     */
    public Properties getResourceAsProperties(String resourcePath) throws ConfigException {
        Properties newProps = null;
        InputStream is = null;
        try {
            is = Config.class.getResourceAsStream(resourcePath);
            if(is == null)
                throw new ConfigException("Resource " + resourcePath + " not found.");
            newProps = new Properties();
            newProps.load(is);
        } catch (IOException e) {
            log.error("caught exception; rethrowing", e);
            throw new ConfigException("Resource file " + resourcePath + " not found", e);
        } finally {
            try {
                if(is != null)
                    is.close();
            } catch (IOException e) {
                log.warn("caught exception while closing InputStream; ignoring", e);
            }
        }
        return newProps;
    }


    /**
     * Load default resource file into configuration properties.
     * @throws IOException when properties file is not found
     */
    public synchronized void load() throws ConfigException {
        load(DEFAULT_PROPERTY_FILE_RESOURCE_PATH);
    }

    /**
     * Load resource file into configuration properties.
     * @throws ConfigException when properties file is not found
     */
    public synchronized void load(String propertyFileResourcePath) throws ConfigException {
        loadWithPrefix(propertyFileResourcePath, "");
    }

    /**
     * Load default resource file into configuration properties adding specified prefix
     * to property names.
     * @throws IOException when properties file is not found
     */
    public synchronized void loadWithPrefix(String propertyNamePrefix) throws ConfigException {
        loadWithPrefix(DEFAULT_PROPERTY_FILE_RESOURCE_PATH, propertyNamePrefix);
    }

    /**
     * Load resource file into configuration properties adding specified prefix
     * to property names.
     * @throws ConfigException when properties file is not found
     */
    public synchronized void loadWithPrefix(String propertyFileResourcePath, String propertyNamePrefix) throws ConfigException {
        log.info("Loading configuration properties from resource path " + propertyFileResourcePath);

        // check if prefix is null and replace it with empty string
        if(propertyNamePrefix == null)
            propertyNamePrefix = "";

        // load properties
        Properties newProps = getResourceAsProperties(propertyFileResourcePath);

        // copy new properties to global config properties
        if(!newProps.isEmpty()) {

            // if necessary, create global config properties
            if(_props == null)
                _props = new Properties();

            // copy new keys using prefix
            for(Object key : newProps.keySet()) {
                _props.setProperty(propertyNamePrefix + key, (String) newProps.get(key));
            }
        }

        // log configuration properties contents
        if(log.isDebugEnabled()) {
            log.debug("Configuration properties contents:");
            Set<Object> keySet = _props.keySet();
            for(Object key : keySet)
                log.debug(key + " " + _props.get(key));
        }
    }

}
